/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_CAMERADRIVINGPERSERVICE_CAMERADRIVINGPERCEPTIONSERVICE_SKELETON_H
#define JAC_AP_CAMERADRIVINGPERSERVICE_CAMERADRIVINGPERCEPTIONSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/cameradrivingperservice/cameradrivingperceptionservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace cameradrivingperservice {
namespace skeleton {
namespace events
{
    using CameraDrvPerception2FusionEvent = ara::com::internal::skeleton::event::EventAdapter<::perception::vector_CameraPerception2Fusion>;
    static constexpr ara::com::internal::EntityId CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId = 12155U; //CameraDrvPerception2FusionEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class CameraDrivingPerceptionServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultCameraDrvPerception2FusionEvent = CameraDrvPerception2FusionEvent.Initialize();
        ThrowError(resultCameraDrvPerception2FusionEvent);
    }

    CameraDrivingPerceptionServiceSkeleton& operator=(const CameraDrivingPerceptionServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceId, mode)),
              CameraDrvPerception2FusionEvent(ptr->GetSkeleton(), events::CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId, ::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceSpec, mode)),
              CameraDrvPerception2FusionEvent(ptr->GetSkeleton(), events::CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId, ::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceContainer, mode)),
              CameraDrvPerception2FusionEvent(ptr->GetSkeleton(), events::CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId, ::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              CameraDrvPerception2FusionEvent(std::move(other.CameraDrvPerception2FusionEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                CameraDrvPerception2FusionEvent = std::move(other.CameraDrvPerception2FusionEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::CameraDrvPerception2FusionEvent GetCameraDrvPerception2FusionEvent()
        {
            return std::move(CameraDrvPerception2FusionEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = CameraDrvPerception2FusionEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::CameraDrvPerception2FusionEvent CameraDrvPerception2FusionEvent;
    };
    explicit CameraDrivingPerceptionServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceId, mode)),
          CameraDrvPerception2FusionEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId, ::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit CameraDrivingPerceptionServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceSpec, mode)),
          CameraDrvPerception2FusionEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId, ::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit CameraDrivingPerceptionServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceContainer, mode)),
          CameraDrvPerception2FusionEvent(skeletonAdapter->GetSkeleton(), events::CameraDrivingPerceptionServiceCameraDrvPerception2FusionEventId, ::jac::ap::cameradrivingperservice::CameraDrivingPerceptionService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    CameraDrivingPerceptionServiceSkeleton(const CameraDrivingPerceptionServiceSkeleton&) = delete;

    CameraDrivingPerceptionServiceSkeleton(CameraDrivingPerceptionServiceSkeleton&&) = default;
    CameraDrivingPerceptionServiceSkeleton& operator=(CameraDrivingPerceptionServiceSkeleton&&) = default;
    CameraDrivingPerceptionServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          CameraDrvPerception2FusionEvent(token.GetCameraDrvPerception2FusionEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~CameraDrivingPerceptionServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&CameraDrivingPerceptionServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::CameraDrvPerception2FusionEvent CameraDrvPerception2FusionEvent;
};
} // namespace skeleton
} // namespace cameradrivingperservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_CAMERADRIVINGPERSERVICE_CAMERADRIVINGPERCEPTIONSERVICE_SKELETON_H
